#pragma once

/*
* MPIA - GEIRS project
*
* "@(#) $Id: ColorLUTSet.h 795 2020-09-02 11:29:29Z mathar $
*
*/

#include <string>
#include <vector>

#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_URL
#undef PACKAGE_VERSION

using namespace std ;

#define GEIRS_COLORLUTSET_CNT 67

/*!******************************** 
* @brief A list of RGB color lookup tables. 
*  Each contains 256 RGB triples, basically a map from brighnesses
*  to read-green-blue values on a resolution of 1:256.
* @since 2019-11-07
* @author Richard J. Mathar
*/
class ColorLUTSet
{
public:
	/** A flat 2d array of the available colors. The second
	* index from 0 to 255 picks an integer (supposed in th eorder
	* of dark to bright), the first index one of the color schemes.
	* Each integer is a 24 bit represetnation where the lest significant
	* byte is red, the next sign. byte green and the 3rd byte blue.
	vector<int [256]> allLUTs ;
	*/
	static int allLUTs[GEIRS_COLORLUTSET_CNT][256] ;

	/* generate this with  fgrep on slash-star | awk '{print $3}'
	*/
	const char * const names[GEIRS_COLORLUTSET_CNT] = {
		"Grayscale",
		"Hot_Iron_DICOM",
		"PET_DICOM",
		"Hot_Metal_Blue_DICOM",
		"mixed",
		"brown",
		"isocontour",
		"thermal",
		"rgb-green",
		"hot",
		"siemens",
		"cold",
		"003-ice",
		"16_equal",
		"Red",
		"Thallium",
		"nih-image-fire1",
		"heart",
		"brain",
		"001-fire",
		"thal_16",
		"HotRed",
		"royal",
		"grey",
		"gem-256",
		"cequal",
		"invert_gray",
		"32_colors",
		"rgb-blue",
		"nih-image-fire2",
		"auxctq",
		"GreenFire",
		"neon-magenta",
		"rgb-red",
		"pastel",
		"cool",
		"topography",
		"iman",
		"Green",
		"Amber",
		"thal_256",
		"RedHot",
		"HiLo",
		"neon-red",
		"red-blue",
		"5_ramps",
		"002-spectrum",
		"Gold",
		"neon-blue",
		"Siemens",
		"smart",
		"unionjack",
		"20_colors",
		"haze",
		"log_up",
		"hue",
		"log_down",
		"ether",
		"sepia",
		"neon-green",
		"blue_orange",
		"16_colors",
		"thallium",
		"Hot_Iron",
		"blue_orange_icb",
		"HotGreen",
		"gem-16"
		} ;

	ColorLUTSet() ;

	void map(int lutidx, float gray, char rgb[]) const ;
	vector<ushort> tiff(int lutidx, unsigned short * cmp) const ;

protected:

private:

} ; /* ColorLUTSet */
