#pragma once

/*!********************************************
* @file
* @brief API declarations for the Rotflip class.
*/

/*
* MPIA - GEIRS project
*
* "$Header: https://svn.mpia.de/gulli/geirs/src/trunk/Rotflip.h 792 2020-03-16 10:58:44Z mathar $
*
*/

#include <string>
#include <vector>
#include <iostream>

#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_URL
#undef PACKAGE_VERSION

#ifdef HAVE_CCFITS_FITS_H
/** The CCfits library contained in
* <a href="http://heasarc.gsfc.nasa.gov/docs/software/fitsio/ccfits/"> CCfits 2.5</a>
*/
#include <CCfits/CCfits>

using namespace CCfits ;
#endif

using namespace std ;

/*!*************************************
* @brief An operation defined by a rotation by a multiple of 90 degrees and a flip.
* @since 2015-01-24
* @author Richard J. Mathar
*/
class Rotflip {

public:
	/** Number of 90 degree rotations
	*/
	int rot90;

	/** Flip option. 0=none, 1=right-left, 2=up-down.
	*/
	int flipxy;

	Rotflip() ;
	Rotflip(int rot, int flips) ;
#ifdef HAVE_CCFITS_FITS_H
	Rotflip(FITS & ffil) ;
#endif

	Rotflip inverse() const ;
	Rotflip toFits() const ;

protected:
private:
	void normalize() ;

} ; /* Rotflip */

bool operator== (const Rotflip & first , const Rotflip & secnd) ;
bool operator!= (const Rotflip & first , const Rotflip & secnd) ;
