#pragma once

/*!********************************************
* @file
* @brief API declarations for the Subwin class.
*/

/*
* MPIA - GEIRS project
*
* "$Header: https://svn.mpia.de/gulli/geirs/src/trunk/Subwin.h 792 2020-03-16 10:58:44Z mathar $
*
*/

#include <string>
#include <vector>
#include <iostream>

#include "config.h"

#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_URL
#undef PACKAGE_VERSION

#ifdef HAVE_CCFITS_FITS_H
#include <CCfits/CCfits>
#endif

#include "Rotflip.h"

using namespace std ;
#ifdef HAVE_CCFITS_FITS_H
using namespace CCfits ;
#endif

/*!*************************************
* @brief A rectangular area defined in standard Cartesian integer coordinates.
* @since 2013-10-30
* @author Richard J. Mathar
*/
class Subwin {

public:
	/** first corner x-coordinate (0-based, 1 less than FITS)
	*/
	int x1;

	/** first corner y-coordinate (0-based, 1 less than FITS)
	*/
	int y1;

	/** Horizontal number of pixels. Width.
	*/
	int xsize;

	/** Vertical number of pixels. Height.
	*/
	int ysize;

	/** Number of pixels along each edge on each H2RG chip.
	* This is 2048.
	*/
	static const int h2rgsize = 2048 ;

	/** Number of pixels along each edge on each H4RG chip.
	* This is 4096.
	*/
	static const int h4rgsize = 4096 ;

	Subwin() ;
	Subwin(int x1, int y1, int width, int height) ;
	Subwin(const string & fts) ;
#ifdef HAVE_CCFITS_FITS_H
	Subwin(ExtHDU & iex) ;
#endif

	int area() const ;
	bool contains(int x, int y) const ;

	Subwin intersection( const Subwin & oth) const ;
	bool inHaw2rgRef(const bool isH4rg) const ;
	Subwin clipRef(const bool isH4rg) const ;
	static bool inHaw2rgRef(int x, int y,const bool isH4rg) ;
	Subwin boundBox( const Subwin & oth) const ;
	Subwin rotflip(int detrot90, int flipxy, const Subwin & parent) const ;
	Subwin rotflip(const Rotflip &rf, const Subwin & parent) const ;
	Subwin rotflipInv(int detrot90, int flipxy, const Subwin & parent) const ;
	Subwin rotflipInv(const Rotflip &rf, const Subwin & parent) const ;
	void rotflipPix(int xy[2], int detrot90, int flipxy) const ;
	void rotflipPix(int xy[2], const Rotflip & rf) const ;
#ifdef UNUSED
	void rotflipPixInv(int xy[2], int detrot90, int flipxy) const ;
#endif

	static vector<Subwin> fullMosaWin(int chipsz, int ndetx, int ndety) ;

	int countSplit(const vector<Subwin> & oth) const ;
	bool crossChip(const int chipsz) const ;
	string datasec(const bool ishaw2rg, const int chipsz) const ;
	string detsec() const ;

	void chipNoxy(const int chipsz, int chipNo[2]) const ;

protected:
	void init(const string & fts) ;
private:

} ; /* Subwin */

bool operator< (const Subwin & first , const Subwin & secnd) ;
bool operator== (const Subwin & first , const Subwin & secnd) ;

ostream & operator<< (ostream & os, const Subwin & win) ;
